;--------------------------------------------------------
; Compteur de consommation
;--------------------------------------------------------

	#include <p16f88.inc>

	;Program Configuration Register 1
	;
	; Code protection off
	; ????
	; In-Circuit Debugger disabled, RB6 and RB7 are general purpose I/O pins
	; Write protection off
	; Code protection off
	; RB3 is digital I/O, HV on MCLR must be used for programming
	; Brown-out Reset Enable bit: BOR disabled
	; RA5/MCLR/VPP pin function is digital I/O, MCLR internally tied to VDD
	; Power-up Timer Enable bit : PWRT enabled
	; Watchdog Timer Enable bit : WDT disabled
	; INTRC oscillator; port I/O function on both RA6/OSC2/CLKO pin and RA7/OSC1/CLKI pin
	__CONFIG _CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_IO

	;Program Configuration Register 2
	;
	; Internal External Switchover bit : disabled
	; Fail-Safe Clock Monitor Enable bit : disabled
	__CONFIG _CONFIG2, _IESO_OFF & _FCMEN_OFF

;--------------------------------------------------------
; Les dclarations
;--------------------------------------------------------

; on active les routines "led" toutes les 100 ms
ACTIONLED EQU 0x64 ; 100
PULSE     EQU 0x2  ; 2 x ACTIONLED soit 200 ms

; Les ports libre service
AN0_IO EQU 0x02
AN1_IO EQU 0x03
AN2_I  EQU 0x05

; les ports d'adresse
PA0 EQU 0x1 ; Lecture : IC: A0
PA1 EQU 0x0 ; Lecture : IC: A1
PA2 EQU 0x7 ; Lecture : IC: A2

; Les port Y
PY0 EQU 0x4
PY1 EQU 0x6

; Les ports X
PX0 EQU 0x7
PX1 EQU 0x0
PX2 EQU 0x6
PX3 EQU 0x5

; Les ports LED
LED_J EQU 0x2
LED_R EQU 0x3

; les ports IC
SDA EQU 0x1 ; SDA: lecture / IC
SCL EQU 0x4 ; SCL: lecture / IC

;--------------------------------------------------------
; Les variables
;--------------------------------------------------------

; premier registre  initialiser  0
INIT_CLR_BANK0_START EQU 0x20

LED_DELAI    EQU 0x20 ; Compte les ms entre 2 activation de la led

PORT         EQU 0x21 ; Contient le n de port scann
PORT_ST      EQU 0x22 ; Contient l'tat du port scann

I2C_ADDR     EQU 0x23 ; Compte les adresses IC

LED_J_CTR    EQU 0x25 ; Compteur du temps d'allumage de la led 1
LED_R_CTR    EQU 0x26 ; Compteur du temps d'allumage de la led 2

PUL_STA0     EQU 0x27 ; contient la mmorisation de l'tat du port
PUL_STA1     EQU 0x28
PUL_STA2     EQU 0x29
PUL_STA3     EQU 0x2a
PUL_STA4     EQU 0x2b
PUL_STA5     EQU 0x2c
PUL_STA6     EQU 0x2d
PUL_STA7     EQU 0x2e

I2C_READ_MEMORY_START EQU 0x34

; quelques octets pour mettre un tag
; ADDR: 0x00
TAG_0        EQU 0x34
TAG_1        EQU 0x35
TAG_2        EQU 0x36
TAG_3        EQU 0x37
TAG_4        EQU 0x38
TAG_5        EQU 0x39
TAG_6        EQU 0x3a
TAG_7        EQU 0x3b
TAG_8        EQU 0x3c
TAG_9        EQU 0x3d
TAG_A        EQU 0x3e
TAG_B        EQU 0x3f
TAG_C        EQU 0x40
TAG_D        EQU 0x41
TAG_E        EQU 0x42
TAG_F        EQU 0x43

; Contient les compteur d'impulsion pour chaque compteur
; ADDR: 0x10
PUL_CPT0     EQU 0x44
PUL_CPT1     EQU 0x45
PUL_CPT2     EQU 0x46
PUL_CPT3     EQU 0x47
PUL_CPT4     EQU 0x48
PUL_CPT5     EQU 0x49
PUL_CPT6     EQU 0x4a
PUL_CPT7     EQU 0x4b

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 0
; ADDR: 0x18
PUL_DELAI0_0 EQU 0x4c
PUL_DELAI0_1 EQU 0x4d
PUL_DELAI0_2 EQU 0x4e
PUL_DELAI0_3 EQU 0x4f

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 1
; ADDR: 0x1c
PUL_DELAI1_0 EQU 0x50
PUL_DELAI1_1 EQU 0x51
PUL_DELAI1_2 EQU 0x52
PUL_DELAI1_3 EQU 0x53

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 2
; ADDR: 0x20
PUL_DELAI2_0 EQU 0x54
PUL_DELAI2_1 EQU 0x55
PUL_DELAI2_2 EQU 0x56
PUL_DELAI2_3 EQU 0x57

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 3
; ADDR: 0x24
PUL_DELAI3_0 EQU 0x58
PUL_DELAI3_1 EQU 0x59
PUL_DELAI3_2 EQU 0x5a
PUL_DELAI3_3 EQU 0x5b

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 4
; ADDR: 0x28
PUL_DELAI4_0 EQU 0x5c
PUL_DELAI4_1 EQU 0x5d
PUL_DELAI4_2 EQU 0x5e
PUL_DELAI4_4 EQU 0x5f

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 5
; ADDR: 0x2c
PUL_DELAI5_0 EQU 0x60
PUL_DELAI5_1 EQU 0x61
PUL_DELAI5_2 EQU 0x62
PUL_DELAI5_3 EQU 0x63

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 6
; ADDR: 0x30
PUL_DELAI6_0 EQU 0x64
PUL_DELAI6_1 EQU 0x65
PUL_DELAI6_2 EQU 0x66
PUL_DELAI6_3 EQU 0x67

; Contient le temps en ms coul entre les
; deux dernire impulsions pour le port 7
; ADDR: 0x34
PUL_DELAI7_0 EQU 0x68
PUL_DELAI7_1 EQU 0x69
PUL_DELAI7_2 EQU 0x6a
PUL_DELAI7_3 EQU 0x6b

; Contient la date relative courante
; ADDR: 0x38
CPT_MS0      EQU 0x6c
CPT_MS1      EQU 0x6d
CPT_MS2      EQU 0x6e
CPT_MS3      EQU 0x6f

TMOUTCTR     EQU 0x70

I2C_READ_MEMORY_STOP EQU 0x71

TMP          EQU 0x70

TMP_MS0      EQU 0x71
TMP_MS1      EQU 0x72
TMP_MS2      EQU 0x73
TMP_MS3      EQU 0x73

TMOUTI2C     EQU 0x74

; dernier registre +1  initialiser  0
INIT_CLR_BANK0_STOP EQU 0x75

INIT_CLR_BANK1_START EQU 0xa0

PUL_LAST0_0 EQU 0xa0
PUL_LAST0_1 EQU 0xa1
PUL_LAST0_2 EQU 0xa2
PUL_LAST0_3 EQU 0xa3

PUL_LAST1_0 EQU 0xa4
PUL_LAST1_1 EQU 0xa5
PUL_LAST1_2 EQU 0xa6
PUL_LAST1_3 EQU 0xa7

PUL_LAST2_0 EQU 0xa8
PUL_LAST2_1 EQU 0xa9
PUL_LAST2_2 EQU 0xaa
PUL_LAST2_3 EQU 0xab

PUL_LAST3_0 EQU 0xac
PUL_LAST3_1 EQU 0xad
PUL_LAST3_2 EQU 0xae
PUL_LAST3_4 EQU 0xaf

PUL_LAST4_0 EQU 0xb0
PUL_LAST4_1 EQU 0xb1
PUL_LAST4_2 EQU 0xb2
PUL_LAST4_3 EQU 0xb3

PUL_LAST5_0 EQU 0xb4
PUL_LAST5_1 EQU 0xb5
PUL_LAST5_2 EQU 0xb6
PUL_LAST5_3 EQU 0xb7

PUL_LAST6_0 EQU 0xb8
PUL_LAST6_1 EQU 0xb9
PUL_LAST6_2 EQU 0xba
PUL_LAST6_3 EQU 0xbb

PUL_LAST7_0 EQU 0xbc
PUL_LAST7_1 EQU 0xbd
PUL_LAST7_2 EQU 0xbe
PUL_LAST7_3 EQU 0xbf

INIT_CLR_BANK1_STOP EQU 0xc0

;--------------------------------------------------------
; reset vector 
;--------------------------------------------------------
STARTUP    code     0x0000
	goto main

;--------------------------------------------------------
; gestionnaire d'interruption
;--------------------------------------------------------
INTERRUPT  code     0x0004

start_of_irq
	                                      ; while (1) {
	;--------------------------
	; on quitte le gestionnaire d'irq
	; uniquement si tous les flags
	; d'irq sont  0
	;--------------------------
	banksel PIR1
	MOVLW (1<<TMR2IF) | (1<<SSPIF)
	ANDWF PIR1,W
	BTFSC STATUS,Z                        ;    if ( PIR1.TMR2IF == 0 && PIR1.SSPIF == 0 )
	RETFIE                                ;       end of irq

	;--------------------------
	; IRQ update counter and scan
	; IRQ update LED
	;--------------------------
	; Teste le flag d'interruption de
	; timer 2. Si le flag n'est pas
	; positionn, on va traiter l'irq
	; suivante. S'il est positionn, on
	; le clear et on traite l'irq
	BTFSS PIR1,TMR2IF                     ;    if ( PIR1.TMR2IF == 1 ) {
	GOTO end_of_ms_irq
	BCF PIR1,TMR2IF                       ;       PIR1.TMR2IF = 0

	; bug ic: on dcremente un timer de 8
	; bits. Toutes les 256ms, on reinitialise
	; l'ic. Si l'ic est utilis on remet
	; le compteur  0.
	; Si il y a une irq, on ne decremente pas.

	; le bit 3 de sspstat est  1 lorsque
	; l'on a recu un start i2c. Lorsqu'il
	; est  1, on enclenche un timeout.

	banksel SSPSTAT
	BTFSS SSPSTAT,3                       ;    if ( SSPSTAT.3 == 1 ) {
	GOTO continue_irq

	banksel TMOUTI2C

	DECFSZ TMOUTI2C,F                     ;       TMOUTI2C --
	GOTO continue_irq                     ;       if ( TMOUTI2C == 0 ) {

	banksel SSPCON

	BCF SSPCON,SSPEN                      ;          reset ic
	BSF SSPCON,SSPEN

	; On incremente le compteur de timeout
	; pour monitorer les erreurs.
	INCF TMOUTCTR,F
	                                      ;       }
continue_irq

	; incremente le compteur de
	; millisecondes
	#include "irq_ms.asm"

	; lance les scans de ports
	#include "irq_scan.asm"

	; lance les routines d'allumage
	; de leds
	#include "irq_led.asm"

	                                      ;    }
end_of_ms_irq

	;--------------------------
	; IRQ IC
	;--------------------------
	; Teste le flag d'interruption de
	; l'IC. Si le flag n'est pas
	; positionn, on recommence. S'il est
	; positionn, on le clear et on traite
	banksel PIR1
	BTFSS PIR1,SSPIF                      ;    if ( PIR1.SSPIF == 1 ) {
	GOTO start_of_irq
	BCF PIR1,SSPIF                        ;       PIR1.SSPIF = 0

	; Procedures IC
	#include "irq_i2c.asm"

	; remise  255 du compteur de timeout
	; ic avant reinitialisation
	CLRF TMOUTI2C                         ;       TMOUTI2C = 0

	                                      ;    }

	; au cas ou une interruption ms soit
	; emise pendant le traitement actuel,
	; on recommence
	GOTO start_of_irq                     ; }

;--------------------------------------------------------
; main code
;--------------------------------------------------------
main

;--------------------------------------------------------
; initialisation
;--------------------------------------------------------
	
	; On va a fond : 8MHz
	banksel OSCCON
	BSF OSCCON,IRCF0 ; bit 4
	BSF OSCCON,IRCF1 ; bit 5
	BSF OSCCON,IRCF2 ; bit 6

	; port b: on positionne les bons bits en I/O
	banksel TRISB
	BSF TRISB,PX0   ; bit 7 1 ; x0: lecture
	BSF TRISB,PX1   ; bit 0 1 ; x1: lecture
	BSF TRISB,PX2   ; bit 6 1 ; x2: lecture
	BSF TRISB,PX3   ; bit 5 1 ; x3: lecture
	BCF TRISB,LED_J ; bit 2 0 ; Led 1: ecriture
	BCF TRISB,LED_R ; bit 3 0 ; Led 2: ecriture
	BSF TRISB,SDA   ; bit 1 1 ; SDA: lecture / IC
	BSF TRISB,SCL   ; bit 4 1 ; SCL: lecture / IC

	; port a: configure digital inputs
	banksel ANSEL
	CLRF ANSEL

	; port a: on positionne les bons bits en I/O
	banksel TRISA
	BSF TRISA,PA0    ; bit 1 1 ; Lecture : IC: A0
	BSF TRISA,PA1    ; bit 0 1 ; Lecture : IC: A1
	BSF TRISA,PA2    ; bit 7 1 ; Lecture : IC: A2
	BCF TRISA,PY0    ; bit 4 0 ; Ecriture: Y0
	BCF TRISA,PY1    ; bit 6 0 ; Ecriture: Y1
	BCF TRISA,AN0_IO ; bit 2 0 ; Lecture: I/O optionnel
	BCF TRISA,AN1_IO ; bit 3 0 ; Lecture: I/O optionnel
	BSF TRISA,AN2_I  ; bit 5 1 ; Lecture: I optionnel (pas le choix)

	; on positonne le port B en pull up
	banksel OPTION_REG
	BCF OPTION_REG,NOT_RBPU



	;------------------------------------
	; Configuration du port IC
	;------------------------------------
	; Bus I2C, on emule un 24C02. Son adresse est
	;         1010xxx - 0101 0xxx - 0x50.
	; Les 3 derniers bits sont lus via PA0..2
	; On va lire les bits de conf, et on positionne 
	; l'adresse dans W
	banksel PORTA
	MOVLW 0xA0 ; 0x50 << 1 ; le bit read/write
	BTFSC PORTA,1
	IORLW 0x02
	BTFSC PORTA,0
	IORLW 0x04
	BTFSC PORTA,7
	IORLW 0x08
	banksel SSPADD
	MOVWF SSPADD

	; reset du flag d'interruption IC
	banksel PIR1
	BCF PIR1,SSPIF

	; reset du buffer stats. Tout doit tre
	; mis  0 pour un bon fonctionnement.
	banksel SSPSTAT
	CLRF SSPSTAT

	banksel SSPCON
	BCF SSPCON,WCOL  ; No collision
	BCF SSPCON,SSPOV ; No overflow
	BSF SSPCON,SSPEN ; Enables the serial port and configures the
	                 ; SDA and SCL pins as serial port pins
	BSF SSPCON,CKP   ; Enable clock
	BCF SSPCON,SSPM3 ; 0110 = I2C Slave mode, 7-bit address
	                 ; Apparemment les IRQ sont fournies de base
	                 ; Le mode irq sur start / stop fourni juste des IRQ en plus.
	BSF SSPCON,SSPM2 ;
	BSF SSPCON,SSPM1 ;
	BCF SSPCON,SSPM0 ;

	; On lit le buffer de reception IC pour faire
	; un reset du flag BF.
	banksel SSPBUF
	MOVFW SSPBUF

	; Activation de l'interruption
	banksel PIE1
	BSF PIE1,SSPIE




	;------------------------------------
	; timer 2
	;------------------------------------
	; On veut une IRQ toutes les ms.
	; Le premier diviseur est cadenc  2MHz 
	; (Fosc/4 => 8MHz / 4) on divise par 4,
	; cela fait 500kHz. 1ms * 125kHz => 500.
	; Si on met le postscaler a 4, on a 
	; 500 / 4 = 125. Il faut placer 125 dans
	; le compteur, et mettre le postcaler  4
	;
	banksel PR2
	; charge 125 dans le registre compar
	MOVLW 0x7d ; 125
	MOVWF PR2
	banksel T2CON
	; - prescaler a 4
	BSF T2CON,T2CKPS0 ; bit 0 : 1
	BCF T2CON,T2CKPS1 ; bit 1 : 0
	; - On allume le compteur
	BSF T2CON,TMR2ON  ; bit 2 : 1
	; - postscaler a 4
	BSF T2CON,TOUTPS0 ; bit 3 : 1
	BSF T2CON,TOUTPS1 ; bit 4 : 1
	BCF T2CON,TOUTPS2 ; bit 5 : 0
	BCF T2CON,TOUTPS3 ; bit 6 : 0

	; clear irq flag
	banksel PIR1
	BCF PIR1,TMR2IF

	; - On positionne l'IRQ
	banksel PIE1
	BSF PIE1,TMR2IE

	;------------------------------------
	; clear memory
	;------------------------------------

	; on va initialiser  0 tous les
	; registres de INIT_CLR_START 
	; INIT_CLR_STOP.
	;
	; On accde indirectement 
	; chaque registre
	BCF STATUS,IRP                        ; STATUS.IRP = 0
	MOVLW INIT_CLR_BANK0_START
	MOVWF FSR                             ; FSR = INIT_CLR_BANK0_START

	                                      ; do {
init_clear_bank0

	; on clear le registre point
	CLRF INDF                             ;    *FSR = 0

	; on passe au registre suivant
	INCF FSR,F                            ;    FSR++

	; On vrifie si l'on a atteint le
	; dernier registre. On charge la
	; dernire valeur attendue + 1, on
	; fait un ou exclusif. Si les deux
	; valeurs sont identiques, le rsultat
	; vaudra 0 et Z sera positionn
	MOVLW INIT_CLR_BANK0_STOP
	XORWF FSR,W
	BTFSS STATUS,Z                        ; } while ( FSR != INIT_CLR_BANK0_STOP )
	GOTO init_clear_bank0

	;
	; Meme chose pour bank1
	;
	MOVLW INIT_CLR_BANK1_START
	MOVWF FSR                             ; FSR = INIT_CLR_BANK1_START
	                                      ; do {
init_clear_bank1
	CLRF INDF                             ;    *FSR = 0
	INCF FSR,F                            ;    FSR++
	MOVLW INIT_CLR_BANK1_STOP
	XORWF FSR,W
	BTFSS STATUS,Z                        ; } while ( FSR != INIT_CLR_BANK1_STOP )
	GOTO init_clear_bank1

	;------------------------------------
	; Init variables
	;------------------------------------
	banksel ACTIONLED

	; positionne le dlai d'attente entre
	; deux traitement de l'allumage des
	; leds
	MOVLW ACTIONLED
	MOVWF LED_DELAI                       ; LED_DELAI = ACTIONLED

	; positionne le pointeur sur la
	; premire adresse i2c  renvoyer
	CLRF I2C_ADDR                         ; I2C_ADDR = 0

	;------------------------------------
	; Charge le code contenant le tag de
	; la release du programme
	;------------------------------------
	#include "tag.asm"

	;------------------------------------
	; teint les leds
	;------------------------------------
	banksel PORTB
	BSF PORTB,LED_J
	BSF PORTB,LED_R

	;------------------------------------
	; Activation des interruptions
	;------------------------------------
	banksel INTCON
	BSF INTCON,GIE
	BSF INTCON,PEIE

;--------------------------------------------------------
; programme principal:
; Le programme est entirement vnementiel, le
; programme principal ne fait rien.
;--------------------------------------------------------
do_sleep
	GOTO do_sleep

;--------------------------------------------------------
; include functions
;--------------------------------------------------------

	#include "pulse_treat.asm"

end
