;-----------------------------------------
;
; scan des ports
;
; PORT sert a indiquer le n de port qui vient d'tre scann
; PORT_ST vaut 1 si une impulsion est en cours
;
;-----------------------------------------


;         Vcc
;
;      ^  ^  ^  ^
;      |  |  |  |      Y0          Y1
;      #  #  #  #
;      #  #  #  #      |            |
;      |  |  |  |      |\ p0        |\ p4
; X0 --+-----------------\------------\---
;         |  |  |      |            |
;         |  |  |      |\ p1        |\ p5
; X1 -----+--------------\------------\---
;            |  |      |            |
;            |  |      |\ p2        |\ p6
; X2 --------+-----------\------------\---
;               |      |            |
;               |      |\ p3        |\ p7
; X3 -----------+--------\------------\---
;                      |            |
;
;
;                Vcc
;
;                 ^
;                 |
;                ###
;                ###
;                ###
;                 |
;                 +----------------+--- X0
;                 |                |
;                 |                |
;            #   /            #   /
;            # /              # /
;       p0 --#           p4 --#
;            # \              # \
;            #   \            #   \
;                \/               \/
;                 |                |
;                 |                |
;
;                 Y0               Y1
;
;
;   Y0 = GND and Y1 = Vcc
;
;         p0..3   p4..7   x0..3
;         open    open    Vcc
;         open    closed  Vcc
;         closed  open    GND
;         closed  closed  GND
;
;   Y0 = Vcc and Y1 = GND
;
;         p0..3   p4..7   x0..3
;         open    open    Vcc
;         open    closed  GND
;         closed  open    Vcc
;         closed  closed  GND
;

	banksel   PORTA ; PORTA, PORTB, PORT_ST et PORT sont dans la mme banque

	; on efface le registre qui va
	; contenir le statut d'impulsion
	CLRF      PORT_ST                     ; PORT_ST = 0

	; On positionne Y0  0 et Y1  1
	; pour rcuprer les valeurs des
	; connecteurs p0..3
	BCF       PORTA,PY0                   ; PORTA.PY0 = 0
	BSF       PORTA,PY1                   ; PORTA.PY1 = 1

	; maintenant, on va scanner les
	; rsultats sur X0..3 qui
	; correspondent aux connecteurs
	; p0..3

	; on met le n de port que l'on lit.
	; ici le 0. On lit le bit qui va bien
	; dans la matrice. S'il vaut 1, c'est
	; que le bouton n'est pas press on
	; appelle la fonction de traitement
	CLRF      PORT                        ; PORT = 0
	BTFSS     PORTB,PX0                   ; if ( PORTB.PX0 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX1                   ; if ( PORTB.PX1 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX2                   ; if ( PORTB.PX2 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX3                   ; if ( PORTB.PX3 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	; On positionne Y0  1 et Y1  0 pour
	; rcuprer les valeurs des
	; connecteurs p4..7
	BSF       PORTA,PY0                   ; PORTA.PY0 = 1
	BCF       PORTA,PY1                   ; PORTA.PY1 = 0

	; maintenant, on va scanner les
	; rsultats sur X0..3 qui
	; correspondent aux connecteurs
	; p4..7
	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX0                   ; if ( PORTB.PX0 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX1                   ; if ( PORTB.PX1 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX2                   ; if ( PORTB.PX2 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)
	banksel   PORTA

	INCF      PORT,F                      ; PORT++
	BTFSS     PORTB,PX3                   ; if ( PORTB.PX3 == 0 ) {
	BSF       PORT_ST,0                   ;    PORT_ST.0 = 1
	                                      ; }
	CALL      pulse_treat                 ; pulse_treat(PORT, PORT_ST)

