;----------------------------------------------------------
; traitement d'un scan de ports
;
; En entre, on utilise les variables suivantes:
;
;  PORT    : l'index du port que l'on traite compris 
;            entre 0 et 7
;
;  PORT_ST : le statut de l'impulsion. Cette 
;            sera systmatiquement remise  0
;
;----------------------------------------------------------
pulse_treat

	; Cette banque de registres sert pour toute la fonction
	banksel PUL_STA0

	; Toutes les accs indirects sont
	; fait dans les deux premires banques
	BCF STATUS,IRP                        ; STATUS.IRP = 0

	; on slectionne le registre PUL_ST qui
	; va bien le registre a utiliser
	; dpend de l'index du port scann, on
	; va devoir y accder indirectement
	;
	; on prend l'adresse du premier
	; registre PUL_STA0, on y ajoute le
	; numro de port, et on rcupre la
	; valeur dans INDF
	;
	; on aura besoin de pointer sur ce
	; registre quelque soit le rsultat de
	; la comparaison suivante
	MOVLW PUL_STA0
	ADDWF PORT,W
	MOVWF FSR                             ; FSR = PUL_STA0 + PORT

	; a-t-on une impulsion ? Si on a une
	; impulsion, le bit de poids faible
	; de PORT_ST sera  1.
	BTFSC PORT_ST,0                       ; if ( PORT_ST,0 == 0) {
	GOTO on_a_une_impulsion

	; On le met le registre de statut
	; d'impulsion pour le port courant  0.
	; De cette manire, on pourra dtecter
	; une nouvelle impulsion.
	CLRF INDF                             ;    (*FSR).0 = 0
	RETURN                                ;    return
	                                      ; }

on_a_une_impulsion

	; reset du flag d'impulsion
	CLRF PORT_ST                          ; PORT_ST = 0

	; Si on arrive la, c'est que l'on a
	; une impulsion dtecte. Alors on va
	; vrifier tout de suite si on l'a
	; dj vu, et si c'est le cas, on n'a
	; rien a faire, car le traitement de
	; l'impulsion aura dj t fait
	BTFSC INDF,0                          ; if ( (*FSR).0 == 1 ) {
	RETURN                                ;    return
	                                      ; }

	; On a dtect une nouvelle impulsion.
	; On commence par positionner a 1 le
	; registre d'impulsion associ  ce
	; port pour ne pas la traiter plusieurs
	; fois. Aprs cette opration, on n'a
	; plus besoin de pointer sur ce
	; registre.
	BSF INDF,0                            ; (*FSR).0 = 1

	; On fait une petite impulsion
	; lumineuse sur la led. Voir le
	; fichier "irq_led.asm" pour plus de
	; dtails.
	MOVLW PULSE
	MOVWF LED_R_CTR                       ; LED_R_CTR = PULSE

	; On calcule la position en mmoire du
	; compteur d'impulsions associ au
	; port trait. On va y accder
	; indirectement. Ici, STATUS.IRP doit
	; valoir 0
	MOVLW PUL_CPT0
	ADDWF PORT,W
	MOVWF FSR                             ; FSR = PUL_CPT0 + PORT

	; on incrmente le compteur
	; d'impulsions associ au port.
	INCF INDF,F                           ; (*FSR)++

	; on rcupre le pointeur sur l'heure
	; de la dernire impulsion
	BCF STATUS,C
	RLF PORT,W
	MOVWF TMP
	RLF TMP,W
	ADDLW PUL_LAST0_0
	MOVWF FSR                             ; FSR = PUL_LAST0_0 + ( PORT * 4 )

	; on soustrait l'heure de la dernire
	; impulsion  l'heure courante. On
	; stocke a dans le dlai
	;
	; Attention, le test sur la retenue se
	; rapporte  la soustraction prcdente
	; la retenue ne doit pas tre modifie
	; par les instructions intermdiaires
	;
	; Attention, la logique de la retenue est
	; inverse pour la soustraction, elle est
	; mise  0 si il y a overflow
	;
	; TMP_MS[0..3] = CPT_MS[0..3] - *FSR[0..3]
	MOVF INDF,W
	SUBWF CPT_MS0,W
	MOVWF TMP_MS0                         ; TMP_MS0 = CPT_MS0 - *FSR
	MOVF CPT_MS0,W
	MOVWF INDF                            ; *FSR = CPT_MS0
	INCF FSR,F                            ; FSR++
	BTFSS STATUS,C                        ; if ( STATUS.C == 1 ) {
	INCF INDF,F                           ;    (*FSR)++
	                                      ; }

	MOVF INDF,W
	SUBWF CPT_MS1,W
	MOVWF TMP_MS1                         ; TMP_MS1 = CPT_MS1 - *FSR
	MOVF CPT_MS1,W
	MOVWF INDF                            ; *FSR = CPT_MS1
	INCF FSR,F                            ; FSR++
	BTFSS STATUS,C                        ; if ( STATUS.C == 1 ) {
	INCF INDF,F                           ;    (*FSR)++
	                                      ; }

	MOVF INDF,W
	SUBWF CPT_MS2,W
	MOVWF TMP_MS2                         ; TMP_MS2 = CPT_MS2 - *FSR
	MOVF CPT_MS2,W
	MOVWF INDF                            ; *FSR = CPT_MS2
	INCF FSR,F                            ; FSR++
	BTFSS STATUS,C                        ; if ( STATUS.C == 1 ) {
	INCF INDF,F                           ;    (*FSR)++
	                                      ; }

	MOVF INDF,W
	SUBWF CPT_MS3,W
	MOVWF TMP_MS3                         ; TMP_MS3 = CPT_MS3 - *FSR
	MOVF CPT_MS3,W
	MOVWF INDF                            ; *FSR = CPT_MS3

	; on rcupre le pointeur sur
	; le dlais  mettre a jour
	; TMP contient encore PORT * 2
	BCF STATUS,C
	RLF TMP,W
	ADDLW PUL_DELAI0_0
	MOVWF FSR                             ; FSR = PUL_DELAI0_0 + ( PORT * 4 )

	; on recopie l'heure dans la
	; mmoire d'heure de dernire
	; impulsion

	MOVF TMP_MS0,W
	MOVWF INDF                            ; *FSR = TMP_MS0
	INCF FSR,F                            ; FSR++

	MOVF TMP_MS1,W
	MOVWF INDF                            ; *FSR = TMP_MS1
	INCF FSR,F                            ; FSR++

	MOVF TMP_MS2,W
	MOVWF INDF                            ; *FSR = TMP_MS2
	INCF FSR,F                            ; FSR++

	MOVF TMP_MS3,W
	MOVWF INDF                            ; *FSR = TMP_MS3

	; fin de la fonction
	RETURN
