#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>

struct int4 {
	unsigned char b0; /* lsb */
	unsigned char b1;
	unsigned char b2;
	unsigned char b3; /* msb */
};

struct bytes4 {
	char ident[16];
	unsigned char cpts[8];
	struct int4 tmrs[8];
	struct int4 hour;
	unsigned char tmouti2c;
};

static inline
unsigned int int42int(struct int4 *in)
{
	return ( in->b3 << 24 ) |
	       ( in->b2 << 16 ) |
	       ( in->b1 <<  8 ) |
	       ( in->b0 <<  0 );
}

int main(int argc, char *argv[])
{
	int fd;
	int i;

	struct bytes4 buf;

	unsigned int tmrs[8];
	unsigned int hour;

	/* open i2c device */
	fd = open(argv[1], O_RDONLY);
	if (fd == -1) {
		fprintf(stderr, "open \"%s\" failed: %s\n", argv[1], strerror(errno));
		return 1;
	}

	/* read buffer */
	if (read(fd, &buf, sizeof(buf)) == -1) {
		fprintf(stderr, "read \"%s\" failed: %s\n", argv[1], strerror(errno));
		return 1;
	}

	/* close */
	close(fd);

	/* convert all unsigned int */
	for (i=0; i<8; i++) {
		tmrs[i] = int42int(&buf.tmrs[i]);
	}
	hour = int42int(&buf.hour);

	/* print */
	printf("ident: %s\n", buf.ident);

	for (i=0; i<8; i++)
		printf("cp%d: %u,%u(%fW)\n", i, buf.cpts[i], tmrs[i],
		       tmrs[i] != 0 ? 0.5*3600.0*1000.0/tmrs[i] : 0.0);
	
	printf("date: %u\n", hour);

	printf("timeout i2c: %u\n", buf.tmouti2c);

	return 0;
}
